@@ -7,8 +7,9 @@
 # standard library
 import logging
 import ssl
+import sys
 from datetime import date, datetime
-from email.utils import formatdate
+from email.utils import format_datetime
 from mimetypes import guess_type
 from pathlib import Path
 from typing import Iterable, Tuple
@@ -21,12 +22,18 @@ import markdown
 from git import GitCommandError, GitCommandNotFound, InvalidGitRepositoryError, Repo
 from mkdocs.config.config_options import Config
 from mkdocs.structure.pages import Page
-from mkdocs.utils import get_build_timestamp
+from mkdocs.utils import get_build_datetime
 
 # package
 from mkdocs_rss_plugin import __about__
 from mkdocs_rss_plugin.git_manager.ci import CiHandler
 
+# conditional imports
+if sys.version_info < (3, 9):
+    from mkdocs_rss_plugin.timezoner_pre39 import set_datetime_zoneinfo
+else:
+    from mkdocs_rss_plugin.timezoner_py39 import set_datetime_zoneinfo
+
 # ############################################################################
 # ########## Globals #############
 # ################################
@@ -36,11 +43,6 @@ REMOTE_REQUEST_HEADERS = {
     "User-Agent": "{}/{}".format(__about__.__title__, __about__.__version__),
 }
 
-
-# ############################################################################
-# ########## Globals ###############
-# ##################################
-
 logger = logging.getLogger("mkdocs.mkdocs_rss_plugin")
 
 
@@ -61,11 +63,11 @@ class Util:
             self.git_is_valid = 1
         except InvalidGitRepositoryError as err:
             logging.warning(
-                "[rss-plugin] Path is not a valid git directory. " " Trace: %s" % err
+                f"[rss-plugin] Path '{path}' is not a valid git directory. Trace: {err}"
             )
             self.git_is_valid = 0
         except Exception as err:
-            logging.warning("[rss-plugin] Git issue: %s" % err)
+            logging.warning(f"[rss-plugin] Git issue: {err}")
             self.git_is_valid = 0
 
         # Checks if user is running builds on CI and raise appropriate warnings
@@ -98,23 +100,29 @@ class Util:
         source_date_creation: str = "git",
         source_date_update: str = "git",
         meta_datetime_format: str = "%Y-%m-%d %H:%M",
-    ) -> Tuple[int, int]:
-        """Extract creation and update dates from page metadata (yaml frontmatter) or \
-            git log for given file.
+        meta_default_timezone: str = "UTC",
+        meta_default_time: datetime = None,
+    ) -> Tuple[datetime, datetime]:
+        """Extract creation and update dates from page metadata (yaml frontmatter) or
+        git log for given file.
 
         :param in_page: input page to work with
         :type in_page: Page
-        :param source_date_creation: which source to use (git or meta tag) for creation \
-            date, defaults to "git"
+        :param source_date_creation: which source to use (git or meta tag) for creation
+        date, defaults to "git"
         :type source_date_creation: str, optional
-        :param source_date_update: which source to use (git or meta tag) for update \
-            date, defaults to "git"
+        :param source_date_update: which source to use (git or meta tag) for update
+        date, defaults to "git"
         :type source_date_update: str, optional
         :param meta_datetime_format: datetime string format, defaults to "%Y-%m-%d %H:%M"
         :type meta_datetime_format: str, optional
+        :param meta_default_timezone: timezone to use, defaults to "UTC"
+        :type meta_default_timezone: str, optional
+        :param meta_default_time: time to set if not specified, defaults to None
+        :type meta_default_time: datetime, optional
 
         :return: tuple of timestamps (creation date, last commit date)
-        :rtype: Tuple[int, int]
+        :rtype: Tuple[datetime, datetime]
         """
         # empty vars
         dt_created = dt_updated = None
@@ -124,18 +132,22 @@ class Util:
             dt_created = self.get_date_from_meta(
                 date_metatag_value=in_page.meta.get(source_date_creation),
                 meta_datetime_format=meta_datetime_format,
+                meta_datetime_timezone=meta_default_timezone,
+                meta_default_time=meta_default_time,
             )
             if isinstance(dt_created, str):
-                logger.error(dt_created)
+                logger.error(f"Creation date is a string: {dt_created}")
                 dt_created = None
 
         if source_date_update != "git" and in_page.meta.get(source_date_update):
             dt_updated = self.get_date_from_meta(
                 date_metatag_value=in_page.meta.get(source_date_update),
                 meta_datetime_format=meta_datetime_format,
+                meta_datetime_timezone=meta_default_timezone,
+                meta_default_time=meta_default_time,
             )
             if isinstance(dt_updated, str):
-                logger.error(dt_updated)
+                logger.error(f"Update date is a string: {dt_updated}")
                 dt_updated = None
 
         # explore git log
@@ -159,34 +171,60 @@ class Util:
                     )
             except GitCommandError as err:
                 logging.warning(
-                    "[rss-plugin] Unable to read git logs of '%s'. Is git log readable?"
-                    " Falling back to build date. "
-                    " Trace: %s" % (in_page.file.abs_src_path, err)
+                    f"[rss-plugin] Unable to read git logs of '{in_page.file.abs_src_path}'. "
+                    "Is git log readable? Falling back to build date. "
+                    f"Trace: {err}"
                 )
             except GitCommandNotFound as err:
                 logging.error(
                     "[rss-plugin] Unable to perform command 'git log'. Is git installed? "
                     " Falling back to build date. "
-                    " Trace: %s" % err
+                    f"Trace: {err}"
                 )
                 self.git_is_valid = 0
+            # convert timestamps into datetimes
+            if isinstance(dt_created, (str, float, int)) and dt_created:
+                dt_created = set_datetime_zoneinfo(
+                    datetime.fromtimestamp(float(dt_created)), meta_default_timezone
+                )
+            if isinstance(dt_updated, (str, float, int)) and dt_updated:
+                dt_updated = set_datetime_zoneinfo(
+                    datetime.fromtimestamp(float(dt_updated)), meta_default_timezone
+                )
         else:
             pass
 
         # return results
         if all([dt_created, dt_updated]):
             return (
-                int(dt_created),
-                int(dt_updated),
+                dt_created,
+                dt_updated,
+            )
+        elif dt_created:
+            logger.info(
+                f"[rss-plugin] Updated date could not be retrieved for page: "
+                f"{in_page.file.abs_src_path}. Maybe it has never been committed yet?"
+            )
+            return (
+                dt_created,
+                get_build_datetime(),
+            )
+        elif dt_updated:
+            logger.info(
+                f"[rss-plugin] Creation date could not be retrieved for page: "
+                f"{in_page.file.abs_src_path}. Maybe it has never been committed yet?"
+            )
+            return (
+                get_build_datetime(),
+                dt_updated,
             )
         else:
             logging.warning(
-                "[rss-plugin] Dates could not be retrieved for page: %s."
-                % in_page.file.abs_src_path
+                f"[rss-plugin] Dates could not be retrieved for page: {in_page.file.abs_src_path}."
             )
             return (
-                get_build_timestamp(),
-                get_build_timestamp(),
+                get_build_datetime(),
+                get_build_datetime(),
             )
 
     def get_authors_from_meta(self, in_page: Page) -> Tuple[str] or None:
@@ -207,10 +245,10 @@ class Util:
                 return tuple(in_page.meta.get("author"))
             else:
                 logging.warning(
-                    "[rss-plugin] Type of author value in page.meta (%s) is not valid. "
-                    "It should be str, list or tuple, not: %s."
-                    % in_page.file.abs_src_path,
-                    type(in_page.meta.get("author")),
+                    "[rss-plugin] Type of author value in page.meta "
+                    f"({in_page.file.abs_src_path}) is not valid. "
+                    "It should be str, list or tuple, "
+                    f"not: {type(in_page.meta.get('author'))}."
                 )
                 return None
         elif "authors" in in_page.meta:
@@ -257,8 +295,12 @@ class Util:
         return sorted(output_categories)
 
     def get_date_from_meta(
-        self, date_metatag_value: str, meta_datetime_format: str
-    ) -> float:
+        self,
+        date_metatag_value: str,
+        meta_datetime_format: str,
+        meta_datetime_timezone: str,
+        meta_default_time: datetime,
+    ) -> datetime:
         """Get date from page.meta handling str with associated datetime format and \
             date already transformed by MkDocs.
 
@@ -266,26 +308,35 @@ class Util:
         :type date_metatag_value: str
         :param meta_datetime_format: expected format of datetime
         :type meta_datetime_format: str
+        :param meta_datetime_timezone: timezone to use
+        :type meta_datetime_timezone: str
+        :param meta_default_time: time to set if not specified
+        :type meta_default_time: datetime
 
-        :return: datetime as timestamp
-        :rtype: float
+        :return: datetime
+        :rtype: datetime
         """
         out_date = None
         try:
             if isinstance(date_metatag_value, str):
                 out_date = datetime.strptime(date_metatag_value, meta_datetime_format)
             elif isinstance(date_metatag_value, (date, datetime)):
-                out_date = datetime.combine(date_metatag_value, datetime.min.time())
+                if isinstance(meta_default_time, datetime):
+                    time_to_add = meta_default_time.time()
+                else:
+                    time_to_add = datetime.min.time()
+                out_date = datetime.combine(date_metatag_value, time_to_add)
             else:
                 return "[rss-plugin] Incompatible date type."
         except ValueError as err:
-            return "[rss-plugin] Incompatible date found. Trace: {}".format(err)
+            return f"[rss-plugin] Incompatible date found. Trace: {err}"
         except Exception as err:
-            return "[rss-plugin] Unable to retrieve creation date. Trace: {}".format(
-                err
-            )
+            return f"[rss-plugin] Unable to retrieve creation date. Trace: {err}"
 
-        return out_date.timestamp()
+        if not out_date.tzinfo:
+            out_date = set_datetime_zoneinfo(out_date, meta_datetime_timezone)
+
+        return out_date
 
     def get_description_or_abstract(self, in_page: Page, chars_count: int = 160) -> str:
         """Returns description from page meta. If it doesn't exist, use the \
@@ -306,15 +357,15 @@ class Util:
 
         # If the abstract chars is not unlimited and the description exists,
         # return the description.
-        if description and chars_count != None:
+        if description and chars_count is not None:
             return description
         # If chars count is unlimited, use the html content
-        elif in_page.content and chars_count == None:
-            if chars_count == None or len(in_page.content) < chars_count:
+        elif in_page.content and chars_count is None:
+            if chars_count is None or len(in_page.content) < chars_count:
                 return in_page.content[:chars_count]
         # Use markdown
         elif in_page.markdown:
-            if chars_count == None or len(in_page.markdown) < chars_count:
+            if chars_count is None or len(in_page.markdown) < chars_count:
                 return markdown.markdown(
                     in_page.markdown[:chars_count], output_format="html5"
                 )
@@ -415,9 +466,9 @@ class Util:
             img_length = remote_img.getheader("content-length")
         except (HTTPError, URLError) as err:
             logging.warning(
-                "[rss-plugin] Remote image could not been reached: {}. "
-                "Trying again with GET and disabling SSL verification. Attempt: {}. "
-                "Trace: {}".format(image_url, attempt, err)
+                f"[rss-plugin] Remote image could not been reached: {image_url}. "
+                f"Trying again with GET and disabling SSL verification. Attempt: {attempt}. "
+                f"Trace: {err}"
             )
             if attempt < 2:
                 return self.get_remote_image_length(
@@ -428,12 +479,8 @@ class Util:
                 )
             else:
                 logging.error(
-                    "[rss-plugin] Remote image is not reachable: {} after {} attempts. "
-                    " Trace: {}".format(
-                        image_url,
-                        attempt,
-                        err,
-                    )
+                    f"[rss-plugin] Remote image is not reachable: {image_url} after "
+                    f"{attempt} attempts. Trace: {err}"
                 )
                 return None
 
@@ -516,7 +563,7 @@ class Util:
                     "guid": page.guid,
                     "image": page.image,
                     "link": page.url_full,
-                    "pubDate": formatdate(getattr(page, attribute)),
+                    "pubDate": format_datetime(dt=getattr(page, attribute)),
                     "title": page.title,
                 }
             )
